import logging

from liveconfigs.models.models import ConfigRow

logger = logging.getLogger(__name__)


def config_row_update_or_create(config_name: str, update_fields: dict):
    logger.info('save data to config %s', config_name)
    _, created = ConfigRow.objects.update_or_create(
        name=config_name, defaults=update_fields)
    return f"{config_name} was {'saved to DB' if created else 'updated in DB'}"



from rest_framework import status as http_status
from rest_framework import viewsets
from rest_framework.decorators import action
from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response

from .models import ConfigRow
from .serializers import ConfigRowSerializer


class ConfigRowViewSet(viewsets.ViewSet):
    serializer_class = ConfigRowSerializer
    queryset = ConfigRow.objects.all()
    permission_classes = (IsAuthenticated,)

    @action(methods=['post'], url_name='import_config', url_path='import_config', detail=False)
    def import_config(self, request):
        serializer = self.serializer_class(data=request.data)
        serializer.update_configs()
        return Response(status=http_status.HTTP_200_OK, data={})



# Generated by Django 3.2.20 on 2023-08-18 10:59

import django.contrib.postgres.fields
from django import VERSION
from django.db import migrations, models

if VERSION[0] == 3:
    from django.contrib.postgres.fields.jsonb import JSONField
else:
    from django.db.models import JSONField


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='ConfigRow',
            fields=[
                ('name', models.TextField(primary_key=True, serialize=False)),
                ('value', JSONField(blank=True, null=True)),
                ('topic', models.TextField(blank=True, null=True)),
                ('description', models.TextField(blank=True, null=True)),
                ('tags', django.contrib.postgres.fields.ArrayField(base_field=models.TextField(blank=True, null=True), blank=True, null=True, size=None)),
                ('last_read', models.DateTimeField(blank=True, null=True)),
                ('last_set', models.DateTimeField(blank=True, null=True)),
            ],
        ),
    ]



from liveconfigs.models import BaseConfig


def get_excluded_rows() -> list[str]:
    subclasses = set(BaseConfig.__subclasses__())
    excluded_config_rows = []
    for config_cls in subclasses:
        if isinstance(config_cls.__exported__, list):
            for config_row_name in config_cls.__dict__:
                if not config_row_name.startswith('__') and config_row_name not in config_cls.__exported__:
                    excluded_config_rows.append(config_cls.__prefix__ + config_row_name)
    return excluded_config_rows


def get_actual_config_names() -> list[str]:
    subclasses = set(BaseConfig.__subclasses__())
    actual_configs = []

    for config_cls in subclasses:
        for config_row_name in config_cls.__dict__:
            if not config_row_name.startswith('__'):
                actual_configs.append(config_cls.__prefix__ + config_row_name)

    return actual_configs



import json
from types import UnionType
from django import forms
from django.conf import settings
from django.forms import widgets
from liveconfigs.models import ConfigRow


ENABLE_PRETTY_INPUT = getattr(settings, 'LC_ENABLE_PRETTY_INPUT', False)
MAX_STR_LENGTH_DISPLAYED_AS_TEXTINPUT = getattr(settings, 'LC_MAX_STR_LENGTH_DISPLAYED_AS_TEXTINPUT', 50)

class PrettyJSONEncoder(json.JSONEncoder):
    def __init__(self, *args, indent, sort_keys, **kwargs):
        super().__init__(*args, indent=2, sort_keys=True, **kwargs)


class JSONField(forms.JSONField):
    empty_values = [None, "", ()]


class JSONWidget(widgets.Textarea):
    def format_value(self, value):
        try:
            value = json.dumps(json.loads(value), indent=2, sort_keys=True)
            # these lines will try to adjust size of TextArea to fit to content
            row_lengths = [len(r) for r in value.split('\n')]
            self.attrs['rows'] = min(max(len(row_lengths) + 2, 10), 30)
            self.attrs['cols'] = min(max(max(row_lengths) + 2, 40), 120)
            return value
        except Exception as _:
            return super().format_value(value)


class ConfigRowForm(forms.ModelForm):
    class Meta:
        model = ConfigRow
        fields = "__all__"

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        instance_type = self.instance.registered_row_types.get(self.instance.name)
        if ENABLE_PRETTY_INPUT and not isinstance(instance_type, UnionType):
            max_len = MAX_STR_LENGTH_DISPLAYED_AS_TEXTINPUT
            val = self.instance.value
            if instance_type is bool:
                self.fields["value"] = forms.BooleanField(
                    required=False,
                )
            elif instance_type is int:
                self.fields["value"] = forms.IntegerField(
                    required=False,
                )
            elif instance_type is float:
                self.fields["value"] = forms.FloatField(
                    required=False,
                    widget=forms.NumberInput(attrs={"step": "0.1"}),
                )
            elif instance_type is str:
                if len(val) <= max_len:
                    self.fields["value"] = forms.CharField(
                        required=False, widget=forms.TextInput({"size": max_len})
                    )
            elif isinstance(val, (list, dict)):
                self.fields["value"] = JSONField(encoder=PrettyJSONEncoder)



from rest_framework import serializers

from liveconfigs.models import ConfigRow


class ConfigRowSerializer(serializers.ModelSerializer):
    class Meta:
        model = ConfigRow
        fields = ('name', 'value')

    def update_configs(self):
        for config in self.initial_data:
            config_row, _ = ConfigRow.objects.get_or_create(name=config['name'])
            config_row.value = config['value']
            config_row.save()



from rest_framework.routers import DefaultRouter

from liveconfigs.views import ConfigRowViewSet

router = DefaultRouter()
router.register(prefix='configrow', viewset=ConfigRowViewSet, basename='configrow')
urlpatterns = router.urls



import logging
import typing

from django import VERSION
from django.conf import settings
from django.core import exceptions
from django.db import models
from typeguard import check_type, TypeCheckError, CollectionCheckStrategy
from django.utils import timezone

if VERSION[0] == 3:
    from django.contrib.postgres.fields import JSONField
else:
    from django.db.models import JSONField

logger = logging.getLogger()


class ConfigRow(models.Model):
    """Простая модель для значения одного конфига"""

    name = models.TextField(primary_key=True)
    value = JSONField(blank=True, null=True)
    topic = models.TextField(blank=True, null=True)
    description = models.TextField(blank=True, null=True)
    tags = JSONField(blank=True, null=True)
    last_read = models.DateTimeField(blank=True, null=True)
    last_set = models.DateTimeField(blank=True, null=True)
    default_value = JSONField(blank=True, null=True)
    registered_row_types: dict[str, typing.Any] = dict()
    validators: dict[str, typing.Any] = dict()

    def validate_type(self):
        config_row_type = self.registered_row_types.get(self.name)
        if not config_row_type:
            return
        try:
            check_type(self.value, config_row_type, collection_check_strategy=CollectionCheckStrategy.ALL_ITEMS)
        except TypeCheckError as exc:
            raise exceptions.ValidationError(
                f"Type error of the input value '{self.name}': {exc}"
            )

    def validate_value(self):
        validators = self.validators.get(self.name, None)
        if validators:
            for validator in validators:
                if not validator(self.value):
                    raise exceptions.ValidationError(
                        f"Validation error. Validator '{validator.__name__}', input value '{self.name}': {self.value}"
                    )

    def clean(self):
        self.validate_type()
        self.validate_value()

    def __repr__(self):
        return f"ConfigRow('{self.name}', {self.value})"

    def __str__(self):
        return f"Config '{self.name}' = {self.value}, {self.description or ''}"


class HistoryEvent(models.Model):
    name = models.TextField()
    value = JSONField(blank=True, null=True)
    edit_at = models.DateTimeField(default=timezone.now)
    edit_by = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.CASCADE)



from django.core.management.base import BaseCommand

from liveconfigs.models import ConfigRow
from liveconfigs.utils import get_actual_config_names


class Command(BaseCommand):
    help = 'Удалить конфиги, которые есть в БД, но нет в коде'

    def handle(self, *args, **kwargs):
        actual_configs = get_actual_config_names()
        unused_configs = ConfigRow.objects.exclude(name__in=actual_configs)

        if not unused_configs:
            print("Нет неиспользуемых конфигов")
            return

        names_to_delete = "\n".join([c.name for c in unused_configs])
        print(f"Будут удалены {len(unused_configs)} конфигов:")
        print(names_to_delete)

        ack = input("Вы точно хотите удалить конфиги из списка выше? (y/n)\n")
        if ack == "y":
            unused_configs.delete()
            print("Неиспользуемые конфиги удалены")



from django.contrib.admin import SimpleListFilter
from django.utils.translation import gettext_lazy as _


class JSONFieldListFilter(SimpleListFilter):
    """An admin list filter for ArrayFields."""

    def lookups(self, request, model_admin):
        """Return the filtered queryset."""
        queryset_values = model_admin.model.objects.values_list(
            self.parameter_name, flat=True
        )
        values = []
        for sublist in queryset_values:
            if sublist:
                for value in sublist:
                    if value:
                        values.append((value, value))
            else:
                values.append(("null", "-"))
        return sorted(set(values))

    def get_lookup_next(self, filter_params: dict, lookup: str):
        parameter_name = self.parameter_name
        lookup_next: list | str | None = filter_params.get(parameter_name) or []
        if lookup_next:
            lookup_next = (
                lookup_next[0].split(",")
                if isinstance(lookup_next, list)
                else lookup_next.split(",")
            )
        if lookup in lookup_next:
            lookup_next.remove(lookup)
        else:
            lookup_next.append(lookup)
        return ",".join(lookup_next)

    def choices(self, changelist):
        add_facets = False
        try:
            add_facets = changelist.add_facets
            facet_counts = self.get_facet_queryset(changelist) if add_facets else None
        except AttributeError:
            # в Django < 5.0 нет этого функционала
            pass
        yield {
            "selected": self.value() is None,
            "query_string": changelist.get_query_string(remove=[self.parameter_name]),
            "display": _("All"),
        }

        for i, (lookup, title) in enumerate(self.lookup_choices):
            if add_facets:
                if (count := facet_counts.get(f"{i}__c", -1)) != -1:
                    title = f"{title} ({count})"
                else:
                    title = f"{title} (-)"
            lookup_next = self.get_lookup_next(changelist.get_filters_params(), lookup)

            yield {
                "selected": str(lookup) in (self.value() or []),
                "query_string": (
                    changelist.get_query_string({self.parameter_name: lookup_next})
                    if lookup_next
                    else changelist.get_query_string(remove=[self.parameter_name])
                ),
                "display": title,
            }

    def value(self):
        """
        Return the value (in string format) provided in the request's
        query string for this filter, if any, or None if the value wasn't
        provided.
        """
        _value = self.used_parameters.get(self.parameter_name)
        if _value:
            _value = _value.split(",")
        return _value

    def queryset(self, request, queryset):
        """Return the filtered queryset."""
        lookup_value = self.value()
        if lookup_value:
            lookup_filter = (
                {"{}__isnull".format(self.parameter_name): True}
                if lookup_value == "null"
                else {"{}__has_any_keys".format(self.parameter_name): lookup_value}
            )
            queryset = queryset.filter(**lookup_filter)
        return queryset


class TagsListFilter(JSONFieldListFilter):
    title = "tags"
    parameter_name = "tags"



# Generated by Django 5.0.4 on 2024-05-01 13:40

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('liveconfigs', '0003_configrow_default_value'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='configrow',
            name='tags',
        ),
    ]



import datetime as dt
import logging
import time

from django.conf import settings
from django.core import exceptions

from liveconfigs.models.models import ConfigRow
from liveconfigs.signals import config_row_update_signal

logger = logging.getLogger()

DESCRIPTION_SUFFIX = "_DESCRIPTION"
TAGS_SUFFIX = "_TAGS"
VALIDATORS_SUFFIX = "_VALIDATORS"


CACHE_TTL = getattr(settings, 'LC_CACHE_TTL', 1)


class ConfigRowDescriptor:
    """ Кеширующий дескриптор для работы с конфигами """

    def __init__(self, config_name, default_value, description=None, topic=None, tags=None):
        self.config_name = config_name
        self.default_value = default_value
        self.last_value = default_value
        self.next_check = None
        self.description = description
        self.tags = tags
        self.topic = topic

    def __get__(self, obj, klass=None):
        now = time.time()
        if not self.next_check or now > self.next_check:
            dt_now = dt.datetime.now(tz=dt.timezone.utc)
            try:
                logger.info('accessing db to grab config %s', self.config_name)
                db_row = ConfigRow.objects.get(name=self.config_name)
                update_fields = {}
                if db_row.description != self.description:
                    update_fields['description'] = self.description
                if db_row.tags != self.tags:
                    update_fields['tags'] = self.tags
                if db_row.topic != self.topic:
                    update_fields['topic'] = self.topic
                if db_row.last_read is None or (db_row.last_read < dt_now - dt.timedelta(days=1)):
                    update_fields['last_read'] = dt_now
                if db_row.default_value != self.default_value:
                    update_fields['default_value'] = self.default_value
                if update_fields:
                    config_row_update_signal.send(sender=None, config_name=self.config_name,
                                                  update_fields=update_fields)
                self.last_value = db_row.value
            except exceptions.ObjectDoesNotExist:
                logger.warning('no config %s in db, using default value %s',
                               self.config_name, self.default_value)
                self.last_value = self.default_value
                update_fields = {
                    "name": self.config_name,
                    "value": self.last_value,
                    "description": self.description,
                    "tags": self.tags,
                    "topic": self.topic,
                    "last_read": dt_now,
                    "last_set": dt_now,
                    "default_value": self.default_value,
                }
                config_row_update_signal.send(
                    sender=None, config_name=self.config_name, update_fields=update_fields)

        self.next_check = now + CACHE_TTL
        return self.last_value


class ConfigMeta(type):
    """ Метакласс для конфигов. Подменяет все атрибуты на десктипторы """

    def __new__(cls, name, bases, dct):
        prefix = dct.get('__prefix__', '')
        topic = dct.get('__topic__', name)
        exported = dct.get('__exported__', '__all__')
        if prefix and not prefix.endswith('_'):
            prefix += '_'

        dct['__prefix__'] = prefix
        dct['__topic__'] = topic
        dct['__exported__'] = exported

        validators = dict()
        config_row_types = dict()
        if "__annotations__" in dct:
            config_row_types = dct["__annotations__"]

        for n, v in dct.items():
            if (
                not n.startswith('__')
                and not n.endswith((DESCRIPTION_SUFFIX, TAGS_SUFFIX, VALIDATORS_SUFFIX))
            ):
                if prefix and n in config_row_types:
                    config_row_types[prefix + n] = config_row_types.pop(n)
                dct[n] = ConfigRowDescriptor(prefix + n, v,
                                             description=dct.get(
                                                 n + DESCRIPTION_SUFFIX),
                                             tags=dct.get(n + TAGS_SUFFIX),
                                             topic=topic)
                validators[n] = dct.get(n + VALIDATORS_SUFFIX)

        dct = {
            name: value
            for name, value in dct.items()
            if not name.endswith((DESCRIPTION_SUFFIX, TAGS_SUFFIX, VALIDATORS_SUFFIX))
        }

        ConfigRow.registered_row_types.update(config_row_types)
        ConfigRow.validators.update(validators)
        c = super().__new__(cls, name, bases, dct)
        return c


class BaseConfig(metaclass=ConfigMeta):
    """От этого класса можно наследовать конфиги.
    За значениями этих конфигов система будет обращаться к БД и фоллбечиться
    на значения атрибутов, указаные в самом классе"""



# Generated by Django 5.0.4 on 2024-04-30 11:56

import django.db.models.deletion
import django.utils.timezone
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('liveconfigs', '0001_initial'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='HistoryEvent',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.TextField()),
                ('value', models.JSONField(blank=True, null=True)),
                ('edit_at', models.DateTimeField(default=django.utils.timezone.now)),
                ('edit_by', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]



import datetime as dt
import logging

from django.core import exceptions
from django.core.management.base import BaseCommand

from liveconfigs.models import (DESCRIPTION_SUFFIX, TAGS_SUFFIX,
                                VALIDATORS_SUFFIX, BaseConfig, ConfigRow)

logger = logging.getLogger(__name__)


def save_row(instance_row, reset=False):
    logger.debug(f'save config row {instance_row.config_name}')
    try:
        db_row = ConfigRow.objects.get(name=instance_row.config_name)
        if reset or db_row.value is None:
            db_row.value = instance_row.last_value
            db_row.last_set = dt.datetime.now(tz=dt.timezone.utc)
        db_row.description = instance_row.description
        db_row.tags = instance_row.tags
        db_row.topic = instance_row.topic
        db_row.default_value = instance_row.default_value
        db_row.save()
    except exceptions.ObjectDoesNotExist:
        ConfigRow.objects.create(
            name=instance_row.config_name,
            value=instance_row.last_value,
            description=instance_row.description,
            tags=instance_row.tags,
            topic=instance_row.topic,
            last_set=dt.datetime.now(tz=dt.timezone.utc),
            default_value=instance_row.default_value,
        )


def load_config(reset=False, **kwargs):
    subclasses = set(BaseConfig.__subclasses__())
    for config in subclasses:
        for name, row in config.__dict__.items():
            if not any(
                [name.startswith('__'), name.endswith((DESCRIPTION_SUFFIX, TAGS_SUFFIX, VALIDATORS_SUFFIX))]
            ):
                save_row(row, reset=reset)
        logger.info(f"Config '{config.__name__}' load successfully")


class Command(BaseCommand):
    help = 'Загрузка дефолтных конфигов для админки (существующие параметры не перезаписываются)'

    def handle(self, *args, **kwargs):
        logger.info('Load started')
        load_config(**kwargs)
        logger.info('All configs load successfully')

    def add_arguments(self, parser):
        parser.add_argument(
            '-r',
            '--reset',
            action='store_true',
            default=False,
            help='Сбросить конфиги'
        )



from .descriptors import (DESCRIPTION_SUFFIX, TAGS_SUFFIX, VALIDATORS_SUFFIX,
                          BaseConfig, ConfigMeta, ConfigRowDescriptor)
from .models import ConfigRow, HistoryEvent

__all__ = [
    "BaseConfig", "ConfigMeta", "ConfigRowDescriptor", "ConfigRow", "TAGS_SUFFIX", "DESCRIPTION_SUFFIX",
    "VALIDATORS_SUFFIX", "HistoryEvent"
]



# Generated by Django 5.0.4 on 2024-05-01 13:41

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('liveconfigs', '0004_remove_configrow_tags'),
    ]

    operations = [
        migrations.AddField(
            model_name='configrow',
            name='tags',
            field=models.JSONField(blank=True, null=True),
        ),
    ]



def proper_name(f):
    def _named_generator(*args, **kwargs):
        new_f = f(*args, **kwargs)
        name = getattr(args[0], "__name__", args[0])
        new_f.__name__ = f'{f.__name__}_{"_".join(map(str, (name,) + args[1:]))}'
        return new_f

    return _named_generator


@proper_name
def equal_to(x):
    return lambda y: y == x


@proper_name
def greater_or_equal_than(x):
    return lambda y: y >= x


@proper_name
def greater_than(x):
    return lambda y: y > x


@proper_name
def less_or_equal_than(x):
    return lambda y: y <= x


@proper_name
def less_than(x):
    return lambda y: y < x


def list_of_lists_includes_unique_elements(value):
    """Проверка на уникальность всех элементов внутри структуры типа [['str1', 'str2'], ['str3']]\n
    Внутри могут быть любые типы вместо строк

    Args:
        value: проверяемая структура

    Returns:
        True: Валидация пройдена успешно
        False: Валидация не пройдена
    """
    return len(sum(value, [])) == len(set(sum(value, [])))


@proper_name
def dict_values_are(f: callable) -> callable:
    def _wrapper(x: dict) -> bool:
        return all(map(f, x.values()))

    return _wrapper



import json

from django.conf import settings
from django.contrib import admin
from django.db.models import JSONField
from django.utils import timezone
from import_export import formats, resources
from import_export.admin import ImportExportModelAdmin
from import_export.fields import Field
from tablib import Dataset

from .filters import TagsListFilter
from .forms import ConfigRowForm, JSONWidget
from .models import ConfigRow, HistoryEvent
from .utils import get_excluded_rows


MAX_VISUAL_VALUE_LENGTH = getattr(settings, 'LC_MAX_VISUAL_VALUE_LENGTH', 0)


class ConfigRowResource(resources.ModelResource):
    value_ = Field(column_name='value')

    class Meta:
        model = ConfigRow
        import_id_fields = ('name',)
        fields = ('name', 'value')

    def dehydrate_value(self, cf):
        return cf.value

    def before_import_row(self, row, row_number=None, **kwargs):
        if 'value' in row:
            value = json.dumps(row['value'])
            row['value'] = value

    def import_data_inner(self, dataset: Dataset, dry_run, raise_errors,
                          using_transactions, collect_failed_rows, rollback_on_validation_errors=False, **kwargs):
        excluded_config_rows = get_excluded_rows()
        # dataset содержит все строки конфигов которые были описаны в импортирумом файле
        # в цикле мы удаляем строки, которые не хотели бы импортировать и далее подменяем dataset
        for i, row_name in reversed(list(enumerate(dataset['name']))):
            if row_name in excluded_config_rows:
                del dataset[i]

        return super().import_data_inner(
            dataset, dry_run, raise_errors, using_transactions, collect_failed_rows,
            rollback_on_validation_errors=rollback_on_validation_errors, **kwargs
        )


class ConfigRowAdmin(ImportExportModelAdmin):
    class Media:
        # принудительный перенос длинных значений value чтобы этот столбец не съедал ширину таблицы
        css = {
            'all': ('admin/css/liveconfigs.css',)
        }

    form = ConfigRowForm
    formats = [formats.base_formats.JSON]
    resource_class = ConfigRowResource
    list_display = ('name', 'value_mod', 'is_changed', 'description', 'topic', 'tags', 'last_read', 'last_set')
    fields = ('name', 'value', 'default_value', 'is_changed', 'description', 'topic', 'tags', 'last_read', 'last_set')
    list_filter = ("topic", TagsListFilter,)
    readonly_fields = ('name', 'description', 'topic', 'tags', 'last_read', 'last_set', 'default_value', 'is_changed')
    search_fields = ('name', 'description', 'topic', 'tags')
    actions = ['set_selected_config_rows_to_default']

    def is_changed(self, obj):
        return '▢' if obj.value == obj.default_value else '☑'

    @admin.display(description="value", ordering="value")
    def value_mod(self, obj):
        value = json.dumps(obj.value, ensure_ascii=False)
        if 0 < MAX_VISUAL_VALUE_LENGTH < len(value):
            value = value[:MAX_VISUAL_VALUE_LENGTH//2] + " ... " + value[-MAX_VISUAL_VALUE_LENGTH//2:]
        return value

    def save_model(self, request, obj, form, change):
        user = request.user
        super().save_model(request, obj, form, change)
        HistoryEvent.objects.create(name=obj.name, value=obj.value, edit_at=timezone.now(), edit_by=user)

    def set_selected_config_rows_to_default(self, request, queryset):
        user = request.user
        config_rows, history_events = [], []
        for config_row in queryset:
            config_row.value = config_row.default_value
            history_event = HistoryEvent(name=config_row.name, value=config_row.default_value,
                                         edit_at=timezone.now(), edit_by=user)
            config_rows.append(config_row)
            history_events.append(history_event)
        ConfigRow.objects.bulk_update(config_rows, ["value"])
        HistoryEvent.objects.bulk_create(history_events)


@admin.register(HistoryEvent)
class HistoryEventAdmin(admin.ModelAdmin):
    class Media:
        css = {
            'all': ('admin/css/liveconfigs.css',)
        }

    list_display = ("name", "value_mod", "edit_at", "edit_by")
    fields = ("name", "value", "edit_at", "edit_by")
    readonly_fields = ("name", "edit_at", "edit_by")
    search_fields = ("name", "value_mod")
    list_filter = ("name", "edit_at")
    fake_readonly_fields = ("value", )
    formfield_overrides = {
        JSONField: {'widget': JSONWidget},
    }

    @admin.display(description="value", ordering="value")
    def value_mod(self, obj):
        value = json.dumps(obj.value, ensure_ascii=False)
        if 0 < MAX_VISUAL_VALUE_LENGTH < len(value):
            value = value[:MAX_VISUAL_VALUE_LENGTH//2] + " ... " + value[-MAX_VISUAL_VALUE_LENGTH//2:]
        return value

    def get_form(self, *args, **kwargs):

        form = super().get_form(*args, **kwargs)

        for field_name in self.fake_readonly_fields:
            form.base_fields[field_name].disabled = True

        return form

    def changeform_view(self, request, object_id=None, form_url='', extra_context=None):
        extra_context = extra_context or {}

        extra_context['show_save'] = False
        extra_context['show_save_and_continue'] = False

        return super().changeform_view(request, object_id, form_url, extra_context)

    def has_add_permission(self, request, obj=None):
        return False


admin.site.register(ConfigRow, ConfigRowAdmin)
